@@ -1,6 +1,11 @@
 import functools
 import operator
 import types
+import csv
+import six
+from six.moves import cStringIO
+import codecs
+import os.path
 import sqlalchemy
 import sqlparse
 import prettytable
@@ -8,7 +13,7 @@ from .column_guesser import ColumnGuesserMixin
 
 
 def unduplicate_field_names(field_names):
-    """ Append a number to duplicate field names to make them unique. """
+    """Append a number to duplicate field names to make them unique. """
     res = []
     for k in field_names:
         if k in res:
@@ -19,6 +24,47 @@ def unduplicate_field_names(field_names):
         res.append(k)
     return res
 
+class UnicodeWriter(object):
+    """
+    A CSV writer which will write rows to CSV file "f",
+    which is encoded in the given encoding.
+    """
+
+    def __init__(self, f, dialect=csv.excel, encoding="utf-8", **kwds):
+        # Redirect output to a queue
+        self.queue = cStringIO.StringIO()
+        self.writer = csv.writer(self.queue, dialect=dialect, **kwds)
+        self.stream = f
+        self.encoder = codecs.getincrementalencoder(encoding)()
+
+    def writerow(self, row):
+        _row = [s.encode("utf-8")
+                if hasattr(s, "encode")
+                else s
+                for s in row]
+        self.writer.writerow(_row)
+        # Fetch UTF-8 output from the queue ...
+        data = self.queue.getvalue()
+        data = data.decode("utf-8")
+        # ... and reencode it into the target encoding
+        data = self.encoder.encode(data)
+        # write to the target stream
+        self.stream.write(data)
+        # empty queue
+        self.queue.truncate(0)
+
+    def writerows(self, rows):
+        for row in rows:
+            self.writerow(row)
+
+class CsvResultDescriptor(object):
+    """Provides IPython Notebook-friendly output for the feedback after a ``.csv`` called."""
+    def __init__(self, file_path):
+        self.file_path = file_path
+    def __repr__(self):
+        return 'CSV results at %s' % os.path.join(os.path.abspath('.'), self.file_path)
+    def _repr_html_(self):
+        return '<a href="%s">CSV results</a>' % os.path.join('.', 'files', self.file_path)
     
 class ResultSet(list, ColumnGuesserMixin):
     """
@@ -38,8 +84,8 @@ class ResultSet(list, ColumnGuesserMixin):
                 list.__init__(self, sqlaproxy.fetchmany(size=self.limit))
             else:
                 list.__init__(self, sqlaproxy.fetchall())
-            field_names = unduplicate_field_names(self.keys)
-            self.pretty = prettytable.PrettyTable(field_names)
+            self.field_names = unduplicate_field_names(self.keys)
+            self.pretty = prettytable.PrettyTable(self.field_names)
             if not config.autopandas:
                 for row in self[:config.displaylimit or None]:
                     self.pretty.add_row(row)
@@ -50,7 +96,7 @@ class ResultSet(list, ColumnGuesserMixin):
     def _repr_html_(self):
         if self.pretty:
             result = self.pretty.get_html_string()
-            if self.config.displaylimit and len(self) > config.displaylimit:
+            if self.config.displaylimit and len(self) > self.config.displaylimit:
                 result = '%s\n<span style="font-style:italic;text-align:center;">%d rows, truncated to displaylimit of %d</span>' % (
                     result, len(self), self.config.displaylimit)
             return result
@@ -164,6 +210,26 @@ class ResultSet(list, ColumnGuesserMixin):
         plt.ylabel(self.ys[0].name)
         return plot        
     
+    def csv(self, filename=None, **format_params):
+        """Generate results in comma-separated form.  Write to ``filename`` if given.
+           Any other parameterw will be passed on to csv.writer."""
+        if not self.pretty:
+            return None # no results
+        if filename:
+            outfile = open(filename, 'w')
+        else:
+            outfile = cStringIO.StringIO()
+        writer = UnicodeWriter(outfile, **format_params)
+        writer.writerow(self.field_names)
+        for row in self:
+            writer.writerow(row)
+        if filename:
+            outfile.close()
+            return CsvResultDescriptor(filename)
+        else:
+            return outfile.getvalue()
+        
+    
 def interpret_rowcount(rowcount):
     if rowcount < 0:
         result = 'Done.'
@@ -177,7 +243,7 @@ def run(conn, sql, config, user_namespace):
         for statement in sqlparse.split(sql):
             txt = sqlalchemy.sql.text(statement)
             result = conn.session.execute(txt, user_namespace)
-            if result:
+            if result and config.feedback:
                 print(interpret_rowcount(result.rowcount))
         resultset = ResultSet(result, statement, config)
         if config.autopandas:
